"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const async_enumerable_1 = require("@reactive-js/async-enumerable");
const disposable_1 = require("@reactive-js/disposable");
const http_1 = require("@reactive-js/http");
const observable_1 = require("@reactive-js/observable");
const capabilities_1 = require("./capabilities");
const httpResponseBody_1 = require("./httpResponseBody");
const option_1 = require("@reactive-js/option");
exports.sendHttpRequestUsingFetch = (request) => {
    const { cache, credentials, integrity, method, mode, redirect, referrerPolicy, uri, } = request;
    const url = uri.toString();
    const headers = http_1.httpRequestToUntypedHeaders(request);
    return observable_1.createObservable((subscriber) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const abortController = new AbortController();
        const abortControllerDisposable = disposable_1.createDisposable(() => abortController.abort());
        const bodyEnumerator = async_enumerable_1.identity()
            .enumerateAsync(subscriber, 1)
            .add(subscriber);
        const body = new httpResponseBody_1.HttpResponseBodyImpl(bodyEnumerator);
        subscriber.add(abortControllerDisposable).add(bodyEnumerator);
        try {
            subscriber.dispatch({
                type: 1,
                request,
            });
            const fetchResponse = yield fetch(url, {
                cache,
                credentials,
                headers,
                integrity,
                method,
                mode,
                redirect,
                referrerPolicy,
                signal: abortController.signal,
            });
            const fetchResponseHeaders = fetchResponse.headers;
            const responseHeaders = {};
            fetchResponseHeaders.forEach((v, k) => {
                responseHeaders[k] = v;
            });
            const contentResponse = http_1.parseHttpResponseFromHeaders(fetchResponse.status, responseHeaders, body);
            const response = disposable_1.createDisposableValue(contentResponse, _ => { }).add(subscriber);
            subscriber.dispatch({
                type: 4,
                request,
                response,
            });
            const content = contentResponse === null || contentResponse === void 0 ? void 0 : contentResponse.content;
            if (option_1.isSome(content)) {
                const { contentLength, contentType: { type, subtype, params }, } = content;
                const hasCharset = option_1.isSome(params["charset"]);
                const responseIsText = hasCharset ||
                    type === "text" ||
                    subtype.indexOf("json") >= 0 ||
                    subtype.indexOf("text") >= 0 ||
                    subtype.indexOf("xml") >= 0;
                const responsePromise = responseIsText
                    ? fetchResponse.text.bind(fetchResponse)
                    : contentLength > 0 && capabilities_1.supportsArrayBuffer
                        ? fetchResponse.arrayBuffer.bind(fetchResponse)
                        : capabilities_1.supportsBlob
                            ? fetchResponse.blob.bind(fetchResponse)
                            : () => Promise.reject(new Error("invalid type"));
                const response = yield responsePromise();
                bodyEnumerator.dispatch(response);
                abortControllerDisposable.dispose();
            }
            subscriber.dispose();
        }
        catch (cause) {
            subscriber.dispose({ cause });
        }
    }));
};
//# sourceMappingURL=fetch.js.map