"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const observable_1 = require("@reactive-js/observable");
const pipe_1 = require("@reactive-js/pipe");
const blobToString = (blob) => {
    const onSubscribe = (subscriber) => {
        const reader = new FileReader();
        reader.onload = () => {
            subscriber.dispatch(reader.result);
        };
        reader.onerror = () => {
            subscriber.dispose({ cause: reader.error });
        };
        reader.readAsText(blob);
    };
    return observable_1.createObservable(onSubscribe);
};
const blobToArrayBuffer = (body) => {
    const onSubscribe = (subscriber) => {
        const reader = new FileReader();
        reader.onload = () => {
            subscriber.dispatch(reader.result);
        };
        reader.onerror = () => {
            subscriber.dispose({ cause: reader.error });
        };
        reader.readAsArrayBuffer(body);
    };
    return observable_1.createObservable(onSubscribe);
};
const throwTypeError = () => {
    throw new TypeError("invalid type");
};
const bodyToArrayBuffer = (body) => {
    return typeof body === "string" || body instanceof Blob
        ? blobToArrayBuffer(new Blob([body]))
        : body instanceof ArrayBuffer
            ? observable_1.ofValue(body)
            : throwTypeError();
};
const bodyToBlob = (body) => {
    return typeof body === "string" || body instanceof ArrayBuffer
        ? observable_1.ofValue(new Blob([body]))
        : body instanceof Blob
            ? observable_1.ofValue(body)
            : throwTypeError();
};
const bodyToText = (body) => {
    return typeof body === "string"
        ? observable_1.ofValue(body)
        : body instanceof Blob
            ? blobToString(body)
            : body instanceof ArrayBuffer
                ? blobToString(new Blob([body]))
                : throwTypeError();
};
class HttpResponseBodyImpl {
    constructor(body) {
        this.body = body;
    }
    arrayBuffer() {
        return pipe_1.pipe(this.body, observable_1.switchMap(bodyToArrayBuffer));
    }
    blob() {
        return pipe_1.pipe(this.body, observable_1.switchMap(bodyToBlob));
    }
    text() {
        return pipe_1.pipe(this.body, observable_1.switchMap(bodyToText));
    }
}
exports.HttpResponseBodyImpl = HttpResponseBodyImpl;
//# sourceMappingURL=httpResponseBody.js.map