"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const async_enumerable_1 = require("@reactive-js/async-enumerable");
const disposable_1 = require("@reactive-js/disposable");
const http_1 = require("@reactive-js/http");
const observable_1 = require("@reactive-js/observable");
const capabilities_1 = require("./capabilities");
const httpResponseBody_1 = require("./httpResponseBody");
const option_1 = require("@reactive-js/option");
exports.sendHttpRequestUsingXHR = (request) => observable_1.createObservable(subscriber => {
    var _a;
    const xhr = new XMLHttpRequest();
    const xhrSupportsResponseType = "responseType" in xhr;
    const bodyEnumerator = async_enumerable_1.identity()
        .enumerateAsync(subscriber, 1)
        .add(subscriber);
    const body = new httpResponseBody_1.HttpResponseBodyImpl(bodyEnumerator);
    subscriber.add(() => xhr.abort()).add(bodyEnumerator);
    xhr.onerror = () => {
        const cause = new Error("Network request failed");
        subscriber.dispose({ cause });
    };
    xhr.onreadystatechange = () => {
        var _a;
        if (xhr.readyState === 2) {
            const headersRaw = (_a = xhr.getAllResponseHeaders()) !== null && _a !== void 0 ? _a : "";
            const headers = http_1.parseHeaders(headersRaw);
            const contentResponse = http_1.parseHttpResponseFromHeaders(xhr.status, headers, body);
            const response = disposable_1.createDisposableValue(contentResponse, _ => { }).add(subscriber);
            subscriber.add(response);
            subscriber.dispatch({
                type: 4,
                request,
                response,
            });
            const content = contentResponse === null || contentResponse === void 0 ? void 0 : contentResponse.content;
            if (xhrSupportsResponseType && option_1.isSome(content)) {
                const { contentLength, contentType: { type, subtype, params }, } = content;
                const hasCharset = option_1.isSome(params["charset"]);
                const responseIsText = hasCharset ||
                    type === "text" ||
                    subtype.indexOf("json") >= 0 ||
                    subtype.indexOf("text") >= 0 ||
                    subtype.indexOf("xml") >= 0;
                xhr.responseType = responseIsText
                    ? "text"
                    : contentLength > 0 && capabilities_1.supportsArrayBuffer
                        ? "arraybuffer"
                        : capabilities_1.supportsBlob
                            ? "blob"
                            : "";
            }
            else {
                subscriber.dispose();
            }
        }
        else if (xhr.readyState === 4) {
            bodyEnumerator.dispatch(xhr.response);
            subscriber.dispose();
        }
    };
    xhr.onloadstart = () => {
        subscriber.dispatch({
            type: 1,
            request,
        });
    };
    xhr.onprogress = ev => {
        const { loaded: total } = ev;
        if (xhr.readyState === 1) {
            subscriber.dispatch({
                type: 2,
                request,
                total,
            });
        }
    };
    xhr.ontimeout = () => {
        const cause = new Error("Network request failed");
        subscriber.dispose({ cause });
    };
    if (request.credentials === "include") {
        xhr.withCredentials = true;
    }
    else if (request.credentials === "omit") {
        xhr.withCredentials = false;
    }
    xhr.open(request.method, request.uri.toString(), true);
    http_1.writeHttpRequestHeaders(request, (k, v) => xhr.setRequestHeader(k, v));
    xhr.send((_a = request === null || request === void 0 ? void 0 : request.content) === null || _a === void 0 ? void 0 : _a.body);
});
//# sourceMappingURL=xhr.js.map