import { __awaiter } from "tslib";
import { identity } from "@reactive-js/async-enumerable";
import { createDisposable, createDisposableValue, } from "@reactive-js/disposable";
import { httpRequestToUntypedHeaders, parseHttpResponseFromHeaders, } from "@reactive-js/http";
import { createObservable } from "@reactive-js/observable";
import { supportsArrayBuffer, supportsBlob } from "./capabilities";
import { HttpResponseBodyImpl } from "./httpResponseBody";
import { isSome } from "@reactive-js/option";
export const sendHttpRequestUsingFetch = (request) => {
    const { cache, credentials, integrity, method, mode, redirect, referrerPolicy, uri, } = request;
    const url = uri.toString();
    const headers = httpRequestToUntypedHeaders(request);
    return createObservable((subscriber) => __awaiter(void 0, void 0, void 0, function* () {
        const abortController = new AbortController();
        const abortControllerDisposable = createDisposable(() => abortController.abort());
        const bodyEnumerator = identity()
            .enumerateAsync(subscriber, 1)
            .add(subscriber);
        const body = new HttpResponseBodyImpl(bodyEnumerator);
        subscriber.add(abortControllerDisposable).add(bodyEnumerator);
        try {
            subscriber.dispatch({
                type: 1,
                request,
            });
            const fetchResponse = yield fetch(url, {
                cache,
                credentials,
                headers,
                integrity,
                method,
                mode,
                redirect,
                referrerPolicy,
                signal: abortController.signal,
            });
            const fetchResponseHeaders = fetchResponse.headers;
            const responseHeaders = {};
            fetchResponseHeaders.forEach((v, k) => {
                responseHeaders[k] = v;
            });
            const contentResponse = parseHttpResponseFromHeaders(fetchResponse.status, responseHeaders, body);
            const response = createDisposableValue(contentResponse, _ => { }).add(subscriber);
            subscriber.dispatch({
                type: 4,
                request,
                response,
            });
            const content = contentResponse === null || contentResponse === void 0 ? void 0 : contentResponse.content;
            if (isSome(content)) {
                const { contentLength, contentType: { type, subtype, params }, } = content;
                const hasCharset = isSome(params["charset"]);
                const responseIsText = hasCharset ||
                    type === "text" ||
                    subtype.indexOf("json") >= 0 ||
                    subtype.indexOf("text") >= 0 ||
                    subtype.indexOf("xml") >= 0;
                const responsePromise = responseIsText
                    ? fetchResponse.text.bind(fetchResponse)
                    : contentLength > 0 && supportsArrayBuffer
                        ? fetchResponse.arrayBuffer.bind(fetchResponse)
                        : supportsBlob
                            ? fetchResponse.blob.bind(fetchResponse)
                            : () => Promise.reject(new Error("invalid type"));
                const response = yield responsePromise();
                bodyEnumerator.dispatch(response);
                abortControllerDisposable.dispose();
            }
            subscriber.dispose();
        }
        catch (cause) {
            subscriber.dispose({ cause });
        }
    }));
};
//# sourceMappingURL=fetch.js.map