import { ofValue, switchMap, createObservable, } from "@reactive-js/observable";
import { pipe } from "@reactive-js/pipe";
const blobToString = (blob) => {
    const onSubscribe = (subscriber) => {
        const reader = new FileReader();
        reader.onload = () => {
            subscriber.dispatch(reader.result);
        };
        reader.onerror = () => {
            subscriber.dispose({ cause: reader.error });
        };
        reader.readAsText(blob);
    };
    return createObservable(onSubscribe);
};
const blobToArrayBuffer = (body) => {
    const onSubscribe = (subscriber) => {
        const reader = new FileReader();
        reader.onload = () => {
            subscriber.dispatch(reader.result);
        };
        reader.onerror = () => {
            subscriber.dispose({ cause: reader.error });
        };
        reader.readAsArrayBuffer(body);
    };
    return createObservable(onSubscribe);
};
const throwTypeError = () => {
    throw new TypeError("invalid type");
};
const bodyToArrayBuffer = (body) => {
    return typeof body === "string" || body instanceof Blob
        ? blobToArrayBuffer(new Blob([body]))
        : body instanceof ArrayBuffer
            ? ofValue(body)
            : throwTypeError();
};
const bodyToBlob = (body) => {
    return typeof body === "string" || body instanceof ArrayBuffer
        ? ofValue(new Blob([body]))
        : body instanceof Blob
            ? ofValue(body)
            : throwTypeError();
};
const bodyToText = (body) => {
    return typeof body === "string"
        ? ofValue(body)
        : body instanceof Blob
            ? blobToString(body)
            : body instanceof ArrayBuffer
                ? blobToString(new Blob([body]))
                : throwTypeError();
};
export class HttpResponseBodyImpl {
    constructor(body) {
        this.body = body;
    }
    arrayBuffer() {
        return pipe(this.body, switchMap(bodyToArrayBuffer));
    }
    blob() {
        return pipe(this.body, switchMap(bodyToBlob));
    }
    text() {
        return pipe(this.body, switchMap(bodyToText));
    }
}
//# sourceMappingURL=httpResponseBody.js.map