import { identity } from "@reactive-js/async-enumerable";
import { createDisposableValue } from "@reactive-js/disposable";
import { parseHeaders, parseHttpResponseFromHeaders, writeHttpRequestHeaders, } from "@reactive-js/http";
import { createObservable } from "@reactive-js/observable";
import { supportsArrayBuffer, supportsBlob } from "./capabilities";
import { HttpResponseBodyImpl } from "./httpResponseBody";
import { isSome } from "@reactive-js/option";
export const sendHttpRequestUsingXHR = (request) => createObservable(subscriber => {
    var _a;
    const xhr = new XMLHttpRequest();
    const xhrSupportsResponseType = "responseType" in xhr;
    const bodyEnumerator = identity()
        .enumerateAsync(subscriber, 1)
        .add(subscriber);
    const body = new HttpResponseBodyImpl(bodyEnumerator);
    subscriber.add(() => xhr.abort()).add(bodyEnumerator);
    xhr.onerror = () => {
        const cause = new Error("Network request failed");
        subscriber.dispose({ cause });
    };
    xhr.onreadystatechange = () => {
        var _a;
        if (xhr.readyState === 2) {
            const headersRaw = (_a = xhr.getAllResponseHeaders()) !== null && _a !== void 0 ? _a : "";
            const headers = parseHeaders(headersRaw);
            const contentResponse = parseHttpResponseFromHeaders(xhr.status, headers, body);
            const response = createDisposableValue(contentResponse, _ => { }).add(subscriber);
            subscriber.add(response);
            subscriber.dispatch({
                type: 4,
                request,
                response,
            });
            const content = contentResponse === null || contentResponse === void 0 ? void 0 : contentResponse.content;
            if (xhrSupportsResponseType && isSome(content)) {
                const { contentLength, contentType: { type, subtype, params }, } = content;
                const hasCharset = isSome(params["charset"]);
                const responseIsText = hasCharset ||
                    type === "text" ||
                    subtype.indexOf("json") >= 0 ||
                    subtype.indexOf("text") >= 0 ||
                    subtype.indexOf("xml") >= 0;
                xhr.responseType = responseIsText
                    ? "text"
                    : contentLength > 0 && supportsArrayBuffer
                        ? "arraybuffer"
                        : supportsBlob
                            ? "blob"
                            : "";
            }
            else {
                subscriber.dispose();
            }
        }
        else if (xhr.readyState === 4) {
            bodyEnumerator.dispatch(xhr.response);
            subscriber.dispose();
        }
    };
    xhr.onloadstart = () => {
        subscriber.dispatch({
            type: 1,
            request,
        });
    };
    xhr.onprogress = ev => {
        const { loaded: total } = ev;
        if (xhr.readyState === 1) {
            subscriber.dispatch({
                type: 2,
                request,
                total,
            });
        }
    };
    xhr.ontimeout = () => {
        const cause = new Error("Network request failed");
        subscriber.dispose({ cause });
    };
    if (request.credentials === "include") {
        xhr.withCredentials = true;
    }
    else if (request.credentials === "omit") {
        xhr.withCredentials = false;
    }
    xhr.open(request.method, request.uri.toString(), true);
    writeHttpRequestHeaders(request, (k, v) => xhr.setRequestHeader(k, v));
    xhr.send((_a = request === null || request === void 0 ? void 0 : request.content) === null || _a === void 0 ? void 0 : _a.body);
});
//# sourceMappingURL=xhr.js.map